"use strict";

var nmap = require("nmap");
var DataReader = require("./DataReader");
var SynthDefDecoder2 = require("./SynthDefDecoder2");
var SynthDefDecoder1 = require("./SynthDefDecoder1");

function decodeSynthDef(data) {
  var reader = new DataReader(data);

  if (reader.readInt32() !== 0x53436766) {
    throw new TypeError("synthdef should be start with 'SCgf'");
  }

  var version = reader.readInt32();

  if (version !== 1 && version !== 2) {
    throw new TypeError("invalid version: " + version);
  }

  var numberOfSynthDefs = reader.readInt16();
  var SynthDefDecoder = version === 1 ? SynthDefDecoder1 : SynthDefDecoder2;
  var decoder = new SynthDefDecoder(reader);

  return nmap(numberOfSynthDefs, function () {
    return decoder.decode();
  });
}

module.exports = {
  decode: function decode(data) {
    return decodeSynthDef(data);
  }
};